# Memory Benchmark

The `MemoryBenchmark.ino` was compiled with each `FEATURE_*` and the flash
memory and static RAM sizes were recorded. The `FEATURE_BASELINE` selection is
the baseline, and its memory usage  numbers are subtracted from the subsequent
`FEATURE_*` memory usage.

**Version**: AceTime v1.9.0

**DO NOT EDIT**: This file was auto-generated using `make README.md`.

## How to Regenerate

To regenerate this README.md:

```
$ make clean_benchmarks
$ make benchmarks
$ make README.md
```

The `make benchmarks` target uses `collect.sh` script which calls `auniter.sh`
(https://github.com/bxparks/AUniter) to invoke the Arduino IDE programmatically.
It produces a `*.txt` file with the flash and ram usage information (e.g.
`nano.txt`). It now takes about 16 minutes to generate the `*.txt` files on my
quad-core Intel Core i7-3840QM CPU @ 2.80GHz laptop.

The `make README.md` command calls the `generated_readme.py` Python script which
generates this `README.md` file. The ASCII tables below are generated by the
`generate_table.awk` script, which takes each `*.txt` file and converts it to an
ASCII table.

## Library Size Changes

In v1.3:
* The `BasicZoneManager` and `ExtendedZoneManager` classes were unified under a
  new parent interface `ZoneManager`. This seems to have caused the flash size
  to increase by around 1200 bytes on the AVR processors (Nano, Pro Micro),
  about 500 bytes on a SAMD, about 800 bytes on a ESP8266, 100 bytes on a ESP32,
  and 1400 bytes on a Teensy 3.2. The 8-bit processors suffer the most flash
  size increase proportional to their limited 32kB limit.
* Adding the `ZoneManager` interface simplifies a lot of the complexity with
  saving and restoring time zones using the `TimeZoneData` object, and I think
  it is worth the extra cost of flash size. The mitigating factor is that
  applications targetted towards 8-bit processors will normally have fixed
  number of timezones at compile time, so they can avoid using a `ZoneManager`,
  and avoid this penalty in flash size.

In v1.4.1+:
* Removed the `ZoneInfo::transitionBufSize` field from the `ZoneInfo` struct,
  which saves 1 byte on 8-bit processors (none on 32-bit processors due to
  4-byte alignment). We save 266 bytes for `BasicZoneManager` and 386 bytes for
  `ExtendedZoneManager` when all the zones are loaded into the zone registry.
* Incorporated zoneName compression causes flash/ram usage to increase by
  ~250/120 bytes when using only 1-2 zones, but *decreases* flash consumption by
  1200-2400 bytes when all the zones are loaded into the `ZoneManager`.

In v1.5+:
* Changing `ZoneProcessorCache::getType()` from a `virtual` to a non-virtual
  method saves 250-350 bytes of flash memory when using a `BasicZoneManager` or
  an `ExtendedZoneManager` on an 8-bit AVR processor. Unexpectedly, the flash
  memory consumption *increases* slightly (~0-50 bytes) for some ARM processors
  and the ESP32. Since those processors have far more flash memory, this seems
  like a good tradeoff.
* Changing `BasicZoneProcessor` and `ExtendedZoneProcessor` to be subclasses of
  the templatized `BasicZoneProcessorTemplate` and
  `ExtendedZoneProcessorTemplate` classes causes reduction of flash consumption
  by 250-400 bytes for 32-bit processors. Don't know why. (Very little
  difference for 8-bit AVR).
* Adding a `BrokerFactory` layer of indirection (to support more complex
  ZoneProcessors and Brokers) causes flash memory to go up by 100-200 bytes.

In v1.6:
* Added support for `LinkRegistry` to `BasicZoneManager` and
  `ExtendedZoneManager`. This increases the flash memory usage by 150-500 bytes
  when using one of these classes due to the code required by `LinkRegistrar`. This extra cost is incurred even if the `LinkRegistry` is set to 0 elements. Each `LinkEntry` consumes 8 bytes (2 x `uint32_t`). So a
  `zonedb::kLinkRegistry` with 183 elements uses 1464 extra bytes of flash; a
  `zonedbx::kLinkRegistry` with 207 elements uses 1656 extra bytes.

In v1.7:
* The virtual destructor on the `Clock` base class removed. This reduced the
  flash usage by 618 bytes on AVR processors , 328 bytes on the SAMD21, but only
  50-60 bytes on other 32-bit processors.
* The various `printShortNameTo()` or `printShortTo()` methods changed to
  replace the underscore in the zone names (e.g. `Los_Angeles`) with spaces
  (e.g. `Los Angeles`) to be more human friendly. This made little difference in
  the flash memory consumption, except on the ESP32 where it increased by
  200-300 bytes.

In v1.7.2
* The `SystemClock::clockMillis()` is now non-virtual, using compile-time
  polymorphism through C++ template, and incorporating the same techniques from
  AceRoutine v1.3. Saves about 20-40 bytes of flash.

In v1.7.5:
* `ExtendedZoneProcessor.compareTransitionToMatch()` was modified to
  detect an exact equality between a `Transition` and its `MatchingEra` if any
  of the 3 time stamp versions ('w', 's', 'u') are equal. Adds about 120-150
  bytes of flash on 8-bit and 32-bit processors. But removing
  `originalTransitionTime` from `Transition` decreases flash usage by about 20
  bytes.
* Upgrade ESP8266 Core from 2.7.4 to 3.0.2. Flash consumption increases by
  3-5 kB across the board.
* Upgrade Teensyduino from 1.54 to 1.55. Add memory consumed by `malloc()` and
  `free()` when using classes with virtual methods into baseline
  MemoryBenchmark, reducing the actual memory usage of various features by
  ~3kB.

In v1.8.0:
* Move Clock and SystemClock benchmarks into AceTimeClock v1.0.0.
* Extract thin links from BasicZoneManager and ExtendedZoneManager into
  new BasicLinkManager and ExtendedLinkManager classes.
    * Saves 200-500 bytes of flash for BasicZoneManager and ExtendedZoneManager.
    * Applications can decide whether to use thin links through the LinkManager
      (~2000 flash bytes for AVR) or use fat links through the
      `kZoneAndLinkRegistry` (~5000 flash bytes for AVR).
* Create various test objects as global variables instead of stack variables
  to get a more accurate measurement of their static memory consumption.

In v1.9.0:
* Reduce flash usage of `BasicZoneManager` and `ExtendedZoneManager` by
  1100-1300 bytes on AVR processors:
    * Extract `BasicZoneProcessorCache` and `ExtendedZoneProcessorCache` out
      of `BasicZoneManager` and `ExtendedZoneManager`, making them
      non-templatized.
    * Remove all `virtual` methods from `ZoneManager`, making the ZoneManager
      hierarchy non-polymorphic.
    * Looks like I am reverting some of the changes made in v1.3 when I created
      the `ZoneManager` interface.
* Reduce flash usage of `BasicLinkManager` and `ExtendedLinkManager` by
  68 bytes on AVR processors by removing pure `virtual` methods on `LinkManager`
  base class.
* Increase flash usage by 34 bytes on AVR processors due to slight refactoring
  of `getHighWater()` with `getAllocSize()`. Only 4-8 bytes increase on 32-bit
  processors.

## Arduino Nano

* 16MHz ATmega328P
* Arduino IDE 1.8.16, Arduino CLI 0.19.2
* Arduino AVR Boards 1.8.3

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| baseline                               |    474/   11 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |   1128/   19 |   654/    8 |
| ZonedDateTime                          |   1378/   26 |   904/   15 |
| Manual ZoneManager                     |   1314/   13 |   840/    2 |
|----------------------------------------+--------------+-------------|
| Basic TimeZone (1 zone)                |   6094/  315 |  5620/  304 |
| Basic TimeZone (2 zones)               |   6390/  435 |  5916/  424 |
| BasicZoneManager (1 zone)              |   6300/  326 |  5826/  315 |
| BasicZoneManager (all zones)           |  19042/  702 | 18568/  691 |
| BasicZoneManager (all zones+links)     |  23374/  702 | 22900/  691 |
| BasicLinkManager (all links)           |   2378/   16 |  1904/    5 |
|----------------------------------------+--------------+-------------|
| Extended TimeZone (1 zone)             |   9010/  670 |  8536/  659 |
| Extended TimeZone (2 zones)            |   9402/ 1111 |  8928/ 1100 |
| ExtendedZoneManager (1 zone)           |   9186/  676 |  8712/  665 |
| ExtendedZoneManager (all zones)        |  30952/ 1160 | 30478/ 1149 |
| ExtendedZoneManager (all zones+links)  |  35832/ 1160 | 35358/ 1149 |
| ExtendedLinkManager (all links)        |   2570/   16 |  2096/    5 |
+---------------------------------------------------------------------+

```

## Sparkfun Pro Micro

* 16 MHz ATmega32U4
* Arduino IDE 1.8.16, Arduino CLI 0.19.2
* SparkFun AVR Boards 1.1.13

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| baseline                               |   3470/  153 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |   4100/  159 |   630/    6 |
| ZonedDateTime                          |   4350/  166 |   880/   13 |
| Manual ZoneManager                     |   4308/  153 |   838/    0 |
|----------------------------------------+--------------+-------------|
| Basic TimeZone (1 zone)                |   9048/  453 |  5578/  300 |
| Basic TimeZone (2 zones)               |   9346/  575 |  5876/  422 |
| BasicZoneManager (1 zone)              |   9254/  464 |  5784/  311 |
| BasicZoneManager (all zones)           |  22014/  842 | 18544/  689 |
| BasicZoneManager (all zones+links)     |  26346/  842 | 22876/  689 |
| BasicLinkManager (all links)           |   5352/  158 |  1882/    5 |
|----------------------------------------+--------------+-------------|
| Extended TimeZone (1 zone)             |  11964/  808 |  8494/  655 |
| Extended TimeZone (2 zones)            |  12358/ 1251 |  8888/ 1098 |
| ExtendedZoneManager (1 zone)           |  12140/  814 |  8670/  661 |
| ExtendedZoneManager (all zones)        |  33922/ 1298 | 30452/ 1145 |
| ExtendedZoneManager (all zones+links)  |  38802/ 1298 | 35332/ 1145 |
| ExtendedLinkManager (all links)        |   5544/  158 |  2074/    5 |
+---------------------------------------------------------------------+

```

## SAMD21 M0 Mini

* 48 MHz ARM Cortex-M0+
* Arduino IDE 1.8.16, Arduino CLI 0.19.2
* Sparkfun SAMD Boards 1.8.4

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| baseline                               |  10128/    0 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |  10640/    0 |   512/    0 |
| ZonedDateTime                          |  10728/    0 |   600/    0 |
| Manual ZoneManager                     |  10672/    0 |   544/    0 |
|----------------------------------------+--------------+-------------|
| Basic TimeZone (1 zone)                |  14620/    0 |  4492/    0 |
| Basic TimeZone (2 zones)               |  14900/    0 |  4772/    0 |
| BasicZoneManager (1 zone)              |  14748/    0 |  4620/    0 |
| BasicZoneManager (all zones)           |  31868/    0 | 21740/    0 |
| BasicZoneManager (all zones+links)     |  38468/    0 | 28340/    0 |
| BasicLinkManager (all links)           |  11856/    0 |  1728/    0 |
|----------------------------------------+--------------+-------------|
| Extended TimeZone (1 zone)             |  16516/    0 |  6388/    0 |
| Extended TimeZone (2 zones)            |  16836/    0 |  6708/    0 |
| ExtendedZoneManager (1 zone)           |  16636/    0 |  6508/    0 |
| ExtendedZoneManager (all zones)        |  46244/    0 | 36116/    0 |
| ExtendedZoneManager (all zones+links)  |  53684/    0 | 43556/    0 |
| ExtendedLinkManager (all links)        |  12048/    0 |  1920/    0 |
+---------------------------------------------------------------------+

```

(SAMD compiler does not produce RAM usage numbers.)

## STM32 Blue Pill

* STM32F103C8, 72 MHz ARM Cortex-M3
* Arduino IDE 1.8.16, Arduino CLI 0.19.2
* STM32duino 2.0.0

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| baseline                               |  21420/ 3536 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |  21696/ 3548 |   276/   12 |
| ZonedDateTime                          |  21776/ 3560 |   356/   24 |
| Manual ZoneManager                     |  21700/ 3540 |   280/    4 |
|----------------------------------------+--------------+-------------|
| Basic TimeZone (1 zone)                |  25548/ 3704 |  4128/  168 |
| Basic TimeZone (2 zones)               |  25852/ 3868 |  4432/  332 |
| BasicZoneManager (1 zone)              |  25672/ 3724 |  4252/  188 |
| BasicZoneManager (all zones)           |  42552/ 3724 | 21132/  188 |
| BasicZoneManager (all zones+links)     |  48996/ 3724 | 27576/  188 |
| BasicLinkManager (all links)           |  23132/ 3544 |  1712/    8 |
|----------------------------------------+--------------+-------------|
| Extended TimeZone (1 zone)             |  27240/ 4092 |  5820/  556 |
| Extended TimeZone (2 zones)            |  27556/ 4644 |  6136/ 1108 |
| ExtendedZoneManager (1 zone)           |  27348/ 4100 |  5928/  564 |
| ExtendedZoneManager (all zones)        |  56580/ 4100 | 35160/  564 |
| ExtendedZoneManager (all zones+links)  |  63848/ 4100 | 42428/  564 |
| ExtendedLinkManager (all links)        |  23324/ 3544 |  1904/    8 |
+---------------------------------------------------------------------+

```

An entry of `-1` indicates that the memory usage exceeded the maximum of the
microcontroller and the compiler did not generate the desired information.

## ESP8266

* NodeMCU 1.0, 80MHz ESP8266
* Arduino IDE 1.8.16, Arduino CLI 0.19.2
* ESP8266 Boards 3.0.2

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| baseline                               | 260089/27892 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          | 260545/27908 |   456/   16 |
| ZonedDateTime                          | 260641/27924 |   552/   32 |
| Manual ZoneManager                     | 260557/27896 |   468/    4 |
|----------------------------------------+--------------+-------------|
| Basic TimeZone (1 zone)                | 266169/28644 |  6080/  752 |
| Basic TimeZone (2 zones)               | 266537/28804 |  6448/  912 |
| BasicZoneManager (1 zone)              | 266313/28660 |  6224/  768 |
| BasicZoneManager (all zones)           | 283657/28660 | 23568/  768 |
| BasicZoneManager (all zones+links)     | 290361/28660 | 30272/  768 |
| BasicLinkManager (all links)           | 261933/27904 |  1844/   12 |
|----------------------------------------+--------------+-------------|
| Extended TimeZone (1 zone)             | 268441/29172 |  8352/ 1280 |
| Extended TimeZone (2 zones)            | 268745/29724 |  8656/ 1832 |
| ExtendedZoneManager (1 zone)           | 268569/29180 |  8480/ 1288 |
| ExtendedZoneManager (all zones)        | 298461/29176 | 38372/ 1284 |
| ExtendedZoneManager (all zones+links)  | 306029/29176 | 45940/ 1284 |
| ExtendedLinkManager (all links)        | 262125/27904 |  2036/   12 |
+---------------------------------------------------------------------+

```

## ESP32

* ESP32-01 Dev Board, 240 MHz Tensilica LX6
* Arduino IDE 1.8.16, Arduino CLI 0.19.2
* ESP32 Boards 1.0.6

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| baseline                               | 197748/13084 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          | 199560/13212 |  1812/  128 |
| ZonedDateTime                          | 199708/13228 |  1960/  144 |
| Manual ZoneManager                     | 199656/13204 |  1908/  120 |
|----------------------------------------+--------------+-------------|
| Basic TimeZone (1 zone)                | 204188/13372 |  6440/  288 |
| Basic TimeZone (2 zones)               | 204516/13532 |  6768/  448 |
| BasicZoneManager (1 zone)              | 204368/13388 |  6620/  304 |
| BasicZoneManager (all zones)           | 221664/13388 | 23916/  304 |
| BasicZoneManager (all zones+links)     | 228368/13388 | 30620/  304 |
| BasicLinkManager (all links)           | 200884/13212 |  3136/  128 |
|----------------------------------------+--------------+-------------|
| Extended TimeZone (1 zone)             | 206224/13756 |  8476/  672 |
| Extended TimeZone (2 zones)            | 206512/14308 |  8764/ 1224 |
| ExtendedZoneManager (1 zone)           | 206380/13764 |  8632/  680 |
| ExtendedZoneManager (all zones)        | 236236/13764 | 38488/  680 |
| ExtendedZoneManager (all zones+links)  | 243804/13764 | 46056/  680 |
| ExtendedLinkManager (all links)        | 201076/13212 |  3328/  128 |
+---------------------------------------------------------------------+

```

RAM usage remains constant as more objects are created, which indicates that an
initial pool of a certain minimum size is created regardless of the actual RAM
usage by objects.

## Teensy 3.2

* 96 MHz ARM Cortex-M4
* Arduino IDE 1.8.16, Arduino CLI 0.19.2
* Teensyduino 1.55

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| baseline                               |  10184/ 4152 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |  10512/ 4168 |   328/   16 |
| ZonedDateTime                          |  10572/ 4180 |   388/   28 |
| Manual ZoneManager                     |  10308/ 4160 |   124/    8 |
|----------------------------------------+--------------+-------------|
| Basic TimeZone (1 zone)                |  19396/ 4324 |  9212/  172 |
| Basic TimeZone (2 zones)               |  20188/ 4488 | 10004/  336 |
| BasicZoneManager (1 zone)              |  19656/ 4344 |  9472/  192 |
| BasicZoneManager (all zones)           |  36932/ 4344 | 26748/  192 |
| BasicZoneManager (all zones+links)     |  43636/ 4344 | 33452/  192 |
| BasicLinkManager (all links)           |  11908/ 4160 |  1724/    8 |
|----------------------------------------+--------------+-------------|
| Extended TimeZone (1 zone)             |  23216/ 4712 | 13032/  560 |
| Extended TimeZone (2 zones)            |  24008/ 5264 | 13824/ 1112 |
| ExtendedZoneManager (1 zone)           |  23412/ 4720 | 13228/  568 |
| ExtendedZoneManager (all zones)        |  53300/ 4720 | 43116/  568 |
| ExtendedZoneManager (all zones+links)  |  60868/ 4720 | 50684/  568 |
| ExtendedLinkManager (all links)        |  12100/ 4160 |  1916/    8 |
+---------------------------------------------------------------------+

```

